import numpy as np
from talib import abstract

from xarizmi.candlestick import CandlestickChart
from xarizmi.utils.plot.timeseries.lineplot import lineplot_timeseries


class OBVIndicator:

    def __init__(
        self, candlestick_chart: CandlestickChart, volume: str = "volume"
    ) -> None:
        self.candlestick_chart = candlestick_chart
        self.volume = volume
        self.indicator_data: None | list[float] = None

    def compute(self) -> list[float]:
        close = np.array(
            [candle.close for candle in self.candlestick_chart.candles]
        ).astype(np.float64)
        if self.volume == "volume":
            volume = np.array(
                [candle.volume for candle in self.candlestick_chart.candles]
            ).astype(np.float64)
        elif self.volume == "amount":
            volume = np.array(
                [candle.amount for candle in self.candlestick_chart.candles]
            ).astype(np.float64)
        self.indicator_data = abstract.OBV(close, volume)
        return self.indicator_data.tolist()  # type: ignore

    def plot(
        self,
        fig_size: tuple[int, int] = (10, 6),
        save_path: str | None = None,
        color: str = "blue",
    ) -> None:
        if self.indicator_data is None:
            raise RuntimeError("No data to plot")
        lineplot_timeseries(
            data=self.indicator_data,
            dates_data=[
                candle.datetime for candle in self.candlestick_chart.candles
            ],
            fig_size=fig_size,
            save_path=save_path,
            color=color,
            label="OBV",
            xlabel="Time",
            ylabel="OBV",
            title="On-Balance Volume (OBV)",
        )



import numpy as np
from scipy.signal import argrelextrema  # type: ignore


def find_local_minima_indexes(arr: list[int | float]) -> list[int]:
    """
    Finds the indices of local minima indexes in a list using scipy.

    Parameters:
    arr (list of int/float): The list of values to find local minima in.

    Returns:
    list of int: Indices of local minima.
    """
    return argrelextrema(np.array(arr), np.less_equal)[0].tolist()  # type: ignore  # noqa: E501


def find_local_minima_values(arr: list[int | float]) -> list[int | float]:
    """
    Finds the values of local minima values in a list using scipy.

    Parameters:
    arr (list of int/float): The list of values to find local minima in.

    Returns:
    list of int/float: Values of local minima.
    """
    return [arr[i] for i in find_local_minima_indexes(arr)]


def find_local_maxima_indexes(arr: list[int | float]) -> list[int]:
    """
    Finds the indices of local maxima indexes in a list using scipy.

    Parameters:
    arr (list of int/float): The list of values to find local maxima in.

    Returns:
    list of int: Indices of local maxima.
    """
    return argrelextrema(np.array(arr), np.greater_equal)[0].tolist()  # type: ignore  # noqa: E501


def find_local_maxima_values(arr: list[int | float]) -> list[int | float]:
    """
    Finds the values of local maxima values in a list using scipy.

    Parameters:
    arr (list of int/float): The list of values to find local maxima in.

    Returns:
    list of int/float: Values of local maxima.
    """
    return [arr[i] for i in find_local_maxima_indexes(arr)]



from pydantic import BaseModel
from pydantic import NonNegativeFloat


class Actor(BaseModel):
    """Actor is base class for actors that are involved in the market."""

    asset: NonNegativeFloat = 0

    def _check_funds_is_non_negative(self, funds: float) -> None:
        if funds < 0:
            raise ValueError(f"The given funds '{funds}' is not non-negative")

    def add_funds(self, funds: float) -> None:
        self._check_funds_is_non_negative(funds=funds)
        self.asset += funds

    def withdraw_funds(self, funds: float) -> None:
        self._check_funds_is_non_negative(funds=funds)
        self.asset -= min(funds, self.asset)


class ExchangeActor(Actor):
    """ExchangeActor is an actor that plays the role of Exchange.

    Exchanges usually have a lot of money and assets. They get fee
    per trader and they have the advantage of knowing the orders of
    other actors who use their platform to trade or invest.

    They also have a lot of money and they can manipulate market in
    good or bad ways.
    """


class TraderActor(Actor):
    """A person who is a trader in the market.
    They buy and sell continuously and they hold securities for a
    short period of time.
    """


class InvestorActor(Actor):
    """InvestorActor holds securities for a long time."""
